##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager

from Products.CMFCore.PortalFolder import PortalFolder

from LinkPropertyManager import LinkPropertyManager, InitializeClassLinkPropertyManager
import PropertyContent
# Import permission names
import PropertyCorePermissions

beforeCreate = PropertyContent.beforeCreate

afterCreate = PropertyContent.afterCreate

WorkflowMethod = PropertyContent.WorkflowMethod

def InitializeClassPropertyFolder(klass
                           , view_priviledge
                           , modify_priviledge
                           , list_priviledge
                           , property_extensible_schema__=None
                           , property_link_id_schema__=None
                           ):
    """
    Initialize Class with appropriate class security.
    """
    # Declarative security
    if not hasattr(klass, 'security'):
        klass.security = ClassSecurityInfo()
        klass.security.declareObjectProtected(view_priviledge)
    
    InitializeClassLinkPropertyManager(klass,modify_priviledge,property_extensible_schema__,property_link_id_schema__)

    klass.security.declareProtected(list_priviledge, 'folder_contents')
    
    # Initialize the class, setting up security.
    InitializeClass(klass)
def InitializeClassPropertyContent(klass
                           , view_priviledge
                           , modify_priviledge
                           , property_extensible_schema__=None
                           , property_link_id_schema__=None
                           ):
    """
    Initialize Class with appropriate class security.
    """
    # Declarative security
    if not hasattr(klass, 'security'):
        klass.security = ClassSecurityInfo()
        klass.security.declareObjectProtected(view_priviledge)

    InitializeClassLinkPropertyManager(klass,modify_priviledge,property_extensible_schema__,property_link_id_schema__)
    # Initialize the class, setting up security.
    InitializeClass(klass)

factory_type_information = (
    { 'id'                : 'PropertyFolder'
      , 'meta_type'       : 'CMF PropertyFolder'
      , 'description'     : """\
Property folder object."""
      , 'icon'            : 'pcore_folder_icon.gif'
      , 'product'         : 'CMFPropertyCore'
      , 'factory'         : 'manage_addPropertyFolder'
      , 'filter_content_types' : 0
      , 'immediate_view'  : 'pcore_edit_form'
      , 'actions'         :
#       ( { 'name'          : 'Folder contents'
#           , 'action'      : 'folder_contents'
#           , 'permissions' : (PropertyCorePermissions.ListFolderContents,)
#           , 'category'    : 'folder'
#           }
      ( { 'name'          : 'View'
          , 'action'      : 'pcore_view'
          , 'permissions' : (PropertyCorePermissions.View,)
          , 'category'    : 'folder'
          }
      , { 'name'          : 'Edit'
          , 'action'      : 'pcore_edit_form'
          , 'permissions' : (PropertyCorePermissions.Modify,)
          , 'category'    : 'folder'
          }
      , { 'name'          : 'Metadata'
          , 'action'      : 'metadata_edit_form'
          , 'permissions' : (PropertyCorePermissions.Modify,)
          , 'category'    : 'folder'
          }
      , { 'name'          : 'Local Roles'
          , 'action'      : 'folder_localrole_form'
          , 'permissions' : (PropertyCorePermissions.ManageProperties,)
          , 'category'    : 'folder'
          }
      , { 'name'          : 'Syndication'
          , 'action'      : 'synPropertiesForm'
          , 'permissions' : (PropertyCorePermissions.ManageProperties,)
          , 'category'    : 'folder'
          }
        )
      }
    ,)

def manage_addPropertyFolder(self, id=None, title=None, parent=None, REQUEST=None):
    """
    Create an empty property content object.
    """
    id = beforeCreate(id, 'pcf', REQUEST)
    obj = PropertyFolder(id, title, parent)
    afterCreate(self, id, obj, REQUEST)
    return id

class PropertyFolder(LinkPropertyManager, PortalFolder, PropertyContent.PropertyContent):
    """
    PropertyFolders are property folder objects for the CMF.
    """
    meta_type='CMF PropertyFolder'

    manage_options = PortalFolder.manage_options

    def __init__(self, id, title=None, parent=None
                 , property_extensible_schema__=None
                 , property_link_id_schema__=None):
        PropertyContent.PropertyContent.__init__(self,id,title,parent
                                                 , property_extensible_schema__
                                                 , property_link_id_schema__)
        PortalFolder.__init__(self,id)

    def manage_afterAdd(self, item, container):
        PropertyContent.PropertyContent.manage_afterAdd(self, item, container)
    def manage_beforeDelete(self, item, container):
        PropertyContent.PropertyContent.manage_beforeDelete(self, item, container)

# Cloned from BTreeFolder._replaceFolder
def _replaceFolder(parent, id, klass=PropertyFolder):
    # Replaces a Folder with a klass Folder
    f = parent._getOb(id)
    try:
        title = f.title
    except:
        title = ''
    t = klass(id, title)
    _copyObj(f,t)
    parent._setOb(id, t)

# inner portion of _replaceFolder
def _copyObj(f, t):
    # Copy the contents of the folder to the new folder.
    try:
        ids = f.objectIds()
    except:
        ids = []
    for key, value in f.__dict__.items():
        if key not in ids:
            # Not an ObjectManager item.
            t.__dict__[key] = value
    for key in ids:
        subob = f._getOb(key)
        subob = getattr(subob, 'aq_base', subob)
        t._setOb(key, subob)

# Initialize the PropertyFolder class, setting up security.
InitializeClassPropertyFolder(PropertyFolder
                              , PropertyCorePermissions.View
                              , PropertyCorePermissions.Modify
                              , PropertyCorePermissions.ListFolderContents
                              )
